#line 1 "C:/Users/Sajjad/Downloads/Compressed/Gas & Temp/Gas.c"

sbit LCD_RS at RB7_bit;
sbit LCD_EN at RB6_bit;
sbit LCD_D4 at RB5_bit;
sbit LCD_D5 at RB4_bit;
sbit LCD_D6 at RB3_bit;
sbit LCD_D7 at RB2_bit;

sbit LCD_RS_Direction at TRISB7_bit;
sbit LCD_EN_Direction at TRISB6_bit;
sbit LCD_D4_Direction at TRISB5_bit;
sbit LCD_D5_Direction at TRISB4_bit;
sbit LCD_D6_Direction at TRISB3_bit;
sbit LCD_D7_Direction at TRISB2_bit;


sbit Menu_Key at RC5_bit;
sbit Incr_Key at RC6_bit;
sbit Decr_Key at RC4_bit;


sbit Temp_RLY at RC2_bit;
sbit CO2_RLY at RC3_bit;


signed int Temp,CO2,ShowTimer=0;
unsigned char String[8],Menu=0,Ring;
signed char S_Temp,S_CO2;
bit MKF,IKF,DKF,Display;


void DegreeChar(void) {

 unsigned char i;
 const char Degree[] = {4,10,4,0,0,0,0,0};

 Lcd_Cmd(64);
 for (i=0;i<=7;i++) Lcd_Chr_CP(Degree[i]);
}


void GetTemp(void) {

 signed int Sample1,Sample2;

 Sample1 = ADC_Read(1);
 Sample2 = ADC_Read(3);

 Temp = (Sample1*49)/100 - (Sample2*49)/100;
}


void AlignStrLeft(unsigned char *Array) {

 unsigned char Ring;

 while(((Array[0] < 0x30) || (Array[0] > 0x39)) && (Array[0] != '-') && (Array[0] != '+')) {

 Ring = 0;

 while(Array[Ring] != '\0')
 Array[Ring] = Array[++Ring];
 }
}


void Check_Keys(void) {

 if(Menu_Key) {
 MKF = 1;
 IKF = 0;
 DKF = 0;
 }else if(Incr_Key) {
 MKF = 0;
 IKF = 1;
 DKF = 0;
 }else if(Decr_Key) {
 MKF = 0;
 IKF = 0;
 DKF = 1;
 }
}


void Refresh_Display(void) {

 switch(Menu) {
 case 0:
 GetTemp();
 IntToStr(Temp,String);
 break;
 case 1:
 case 2:
 IntToStr(S_Temp,String);
 }

 if((Menu==1) && !Display)
 Lcd_Out(1,1,"        ");
 else {
 AlignStrLeft(String);
 Lcd_Out(1,1,"T:");
 Lcd_Out_CP(String);
 Lcd_Chr_CP(0);
 Lcd_Out_CP("C ");
 }

 switch(Menu) {
 case 0:
 CO2 = ADC_Read(0)/11;
 IntToStr(CO2,String);
 break;
 case 1:
 case 2:
 IntToStr(S_CO2,String);
 }

 if((Menu==2) && !Display)
 Lcd_Out(2,1,"        ");
 else {
 AlignStrLeft(String);
 Lcd_Out(2,1,"CO2:");
 Lcd_Out_CP(String);
 Lcd_Out_CP("% ");
 }
}


void main() {

 TRISA = 0x0B;
 TRISB = 0x01;
 TRISC = 0xF1;

 PORTA = 0x00;
 PORTB = 0x00;
 PORTC = 0x00;

 ADCON0 = 0x40;
 ADCON1 = 0x04;

 MKF = 0;
 IKF = 0;
 DKF = 0;

 S_Temp = EEPROM_Read(0x00);
 S_CO2 = EEPROM_Read(0x01);

 Lcd_Init();
 Lcd_Cmd(_LCD_CLEAR);
 Lcd_Cmd(_LCD_CURSOR_OFF);
 DegreeChar();
 Delay_ms(1000);

 for(Ring=0;Ring<30;Ring++) {

 Lcd_Out(1,1,"Starting");
 Lcd_Out(2,1,"...");
 Delay_ms(1000);

 Lcd_Cmd(_LCD_CLEAR);
 Delay_ms(1000);

 if((ADC_Read(0)/11)<40) break;
 }

 while(1) {

 ShowTimer++;
 if(ShowTimer >= 400) {
 ShowTimer = 0;
 if(Menu>0) Display = !Display;
 Refresh_Display();
 }

 Check_Keys();

 if(Menu==0) {
 if(CO2>S_CO2) CO2_RLY = 1;
 if(Temp>S_Temp) Temp_RLY = 1;
 }

 if(!Menu_Key && MKF) {
 MKF = 0;
 CO2_RLY = 0;
 Temp_RLY = 0;
 Menu++;
 if(Menu > 2) {
 EEPROM_Write(0x00,S_Temp);
 EEPROM_Write(0x01,S_CO2);
 Menu = 0;
 }
 Refresh_Display();
 }

 if(Menu==1) {
 if(!Incr_Key && IKF) {
 IKF = 0;
 S_Temp++;
 if(S_Temp > 150) S_Temp = -45;
 Refresh_Display();
 }else if(!Decr_Key && DKF) {
 DKF = 0;
 S_Temp--;
 if(S_Temp < -45) S_Temp = 150;
 Refresh_Display();
 }
 }

 if(Menu==2) {
 if(!Incr_Key && IKF) {
 IKF = 0;
 S_CO2++;
 if(S_CO2 > 100) S_CO2 = 0;
 Refresh_Display();
 }else if(!Decr_Key && DKF) {
 DKF = 0;
 S_CO2--;
 if(S_CO2 < 0) S_CO2 = 100;
 Refresh_Display();
 }
 }

 if((!Incr_Key && IKF) || (!Decr_Key && DKF)) {
 IKF = 0;
 DKF = 0;
 CO2_RLY = 0;
 Temp_RLY = 0;
 }
 Delay_ms(1);
 }
}
